/*
 *		XPmath.h
 *
 *	(C) 2003-2005 ramjac Software/Patrick Enoch
 *
 *	floating-point math (no vector)
 *	trig. functions are separated in XPfunctionstrig.h
 *
 *	includes <math.h> in a different namespace
 *	and defines the functions like they are in XPressionist
 */
#ifndef __XP_MATH_
#define __XP_MATH_

/*
 *	fix so we can use "time" as a global variable
 *	we have to exclude "time.h" and "ctime"
 *	which are included through <math.h>
 *	probably because of "timediff" and those functions
 */
#define _TIME_H_
#define _CPP_CTIME 1
#define _GLIBCXX_CTIME 1

namespace imported_math
{
	#include <math.h>
};

#define	pi180	3.1415926535897932385/180.0

inline double abs(double x)		{ return imported_math::fabs(x); 	}

using imported_math::ceil;

inline double deg(double x)		{ return (x*180.0/3.1415926535897932385);	}

using imported_math::exp;

inline double fact(double x)
{
	long	n=(long)x;
	long	result=1;
	for (long i=2; i<=n; i++)
	{
		result=result*i;
	};
	return result;
}
using imported_math::floor;
using imported_math::fmod;
using imported_math::log;
using imported_math::log10;
using imported_math::pow;

inline double rad(double x) 	{ return (x/180.0*3.1415926535897932385);	}

using imported_math::round;

inline double sign(double x)
{
	if (x>0)	{	return 1;	}
	if (x<0)	{	return -1;	}
	return 0;
}

using imported_math::trunc;
using imported_math::sqrt;

/*
 *	"clamp" the value x between min and max
 */
inline double clamp(double min, double max, double val)
{
	if (val > max)	{	return max;	}
	if (val < min)	{	return min;	}
	return val;
}

/*
 *	linear stepping maps
 *
 *	        0 for x<a
 *	f(x) =  linear between 0 and 1 for a < x < b
 *          1 for x>b
 */
inline double linstep(double a, double b, double x)
{
	if (a < b)
	{
		if (x<=a)	{	return 0;	}
		if (x>=b)	{	return 1;	}
		return (x-a)/(b-a);
	}
	else if (b < a)
	{
		if (x<=b)	{	return 1;	}
		if (x>=a)	{	return 0;	}
		return (x-a)/(b-a);
	};
	// a=b
	return 0;
}

/*
 *	smooth stepping uses hemite interpolation
 *
 * 1                            *********
 *                         *****     |
 *                       **          |
 *       a              *            b
 *       |            **
 *       |       *****
 * 0 ************
 *
 */
inline double _smoothstep(double a, double b, double x)
{
	if (a < b)
	{
		if (x<=a)	{	return 0;	}
		if (x>=b)	{	return 1;	}
		return (a-x)*(a-x)*(a-3*b+2*x)/((a-b)*(a-b)*(a-b));
	}
	else if (b < a)
	{
		if (x<=b)	{	return 1;	}
		if (x>=a)	{	return 0;	}
		return (a-x)*(a-x)*(a-3*b+2*x)/((a-b)*(a-b)*(a-b));
	};	
	// a=b
	return 0;
}

double hermite(double arg1, double arg2, double arg3, double arg4, double a)
{
	double B=arg1+arg3;
	double C=arg2-arg4;

	double b=1-a;

	return b*b*b*arg1 + 3*B*b*b*a + 3*C*b*a*a + arg2*a*a*a;
}

#define XPinternal_RAND_MAX 0x7FFF

unsigned long XPinternal_random(void);
void XPinternal_seed(unsigned long seed);

inline double random( double a )
{
	return (a*XPinternal_random()/XPinternal_RAND_MAX);
}

#define XPinternal_dmin(a,b) (((a) < (b)) ? (a) : (b))
#define XPinternal_dmax(a,b) (((a) > (b)) ? (a) : (b))

inline double min( double a, double b )
{
	return XPinternal_dmin(a,b);
}

inline double min( double a, double b, double c )
{
	return XPinternal_dmin(XPinternal_dmin(a,b),c);
}

inline double min( double a, double b, double c, double d )
{
	return XPinternal_dmin(XPinternal_dmin(XPinternal_dmin(a,b),c),d);
}

inline double min( double a, double b, double c, double d, double e )
{
	return XPinternal_dmin(XPinternal_dmin(XPinternal_dmin(XPinternal_dmin(a,b),c),d),e);
}

inline double max( double a, double b )
{
	return XPinternal_dmax(a,b);
}

inline double max( double a, double b, double c )
{
	return XPinternal_dmax(XPinternal_dmax(a,b),c);
}

inline double max( double a, double b, double c, double d )
{
	return XPinternal_dmax(XPinternal_dmax(XPinternal_dmax(a,b),c),d);
}

inline double max( double a, double b, double c, double d, double e )
{
	return XPinternal_dmax(XPinternal_dmax(XPinternal_dmax(XPinternal_dmax(a,b),c),d),e);
}

#endif
