/*
 *		XPvec.h
 *
 *	(C) 2003 ramjac Software/Patrick Enoch
 *
 *	vec is a 3d-vector with components X,Y,Z
 *	which can also be used for colors (R,G,B or H,S,V)
 *
 *	besides the "normal" calculation rules
 *	vec  = vec
 *	vec  = vec * float
 *	vec  = float * vec
 *	vec += vec
 *	vec -= vec
 *	vec *= float
 *	vec /= float
 *	float= vec * vec
 *
 *	we also allow (componentwise)
 *	vec  = vec + float
 *	vec  = vec - float
 *	vec += float
 *	vec -= float
 *	vec  = float
 *	
 */
#ifndef __XP_VEC_
#define __XP_VEC_

#include "XPmath.h"

typedef struct vec
{
inline			vec()
				{
					X=0;	Y=0;	Z=0;
				}
				
inline			vec(const double a, const double b, const double c)
				{
					X=a;	Y=b;	Z=c;
				}

inline			vec(const double arr[])
				{
					X=arr[0];	Y=arr[1];	Z=arr[2];
				}
					
				~vec() {};

	union
	{
		double	X,R,H;
	};
	union
	{
		double	Y,G,S;
	};
	union
	{
		double	Z,B,V;
	};

inline 	vec&	operator = (const vec a)
				{
					X=a.X;	Y=a.Y;	Z=a.Z;
					return *this;
				}
				
inline 	vec&	operator = (const double a)
				{
					X=a;	Y=a;	Z=a;
					return *this;
				}

inline 	vec&	operator += (const double a)
				{
					X+=a;	Y+=a;	Z+=a;
					return *this;
				}

inline 	vec&	operator += (const vec a)
				{
					X+=a.X;	Y+=a.Y;	Z+=a.Z;
					return *this;
				}

inline 	vec&	operator -= (const double a)
				{
					X-=a;	Y-=a;	Z-=a;
					return *this;
				}

inline 	vec&	operator -= (const vec a)
				{
					X-=a.X;	Y-=a.Y;	Z-=a.Z;
					return *this;
				}

inline 	vec&	operator /= (const double a)
				{
					X/=a;	Y/=a;	Z/=a;
					return *this;
				}
	
inline 	vec&	operator *= (const double a)
				{
					X *= a;	Y *= a;	Z *= a;
					return *this;
				}
			
inline 	double	length()
				{
					return sqrt( X*X + Y*Y + Z*Z );
				}
};

inline vec operator + (const vec a, const vec b)
{
	return vec(a.X+b.X , a.Y+b.Y , a.Z+b.Z);
}

inline vec operator - (const vec a, const vec b)
{
	return vec(a.X-b.X , a.Y-b.Y , a.Z-b.Z);
}

inline vec operator + (const vec b, const double a)
{
	return vec(b.X+a , b.Y+a, b.Z+a);
}

inline vec operator - (const vec b, const double a)
{
	return vec(b.X-a , b.Y-a, b.Z-a);
}

inline vec operator * (const double a, const vec b)
{
	return vec(b.X*a, b.Y*a, b.Z*a);
}

inline vec operator * (const vec b, const double a)
{
	return vec(b.X*a, b.Y*a, b.Z*a);
}

inline vec operator / (const vec b, const double a)
{
	return vec(b.X/a, b.Y/a, b.Z/a);
}

inline double operator * (const vec a, const vec b)
{
	return a.X*b.X + a.Y*b.Y + a.Z*b.Z;
}

inline vec operator -(const vec a)
{
	return vec(-a.X, -a.Y, -a.Z);
}

inline bool operator ==(const vec a, const vec b)
{
	return (a.X==b.X && a.Y==b.Y && a.Z==b.Z);
}

inline vec makevec( const double r, const double g, const double b )
{
	return vec(r,g,b);
}

#endif
