/*
 *		XPvec4.h
 *
 *	(C) 2003 ramjac Software/Patrick Enoch
 *
 *	vec4 is a 4d-vector with components X,Y,Z,A
 *	which can also be used for colors (R,G,B,A or H,S,V,A)
 *
 *	besides the "normal" calculation rules
 *	vec4  = vec4
 *	vec4  = vec4 * float
 *	vec4  = float * vec4
 *	vec4 += vec4
 *	vec4 -= vec4
 *	vec4 *= float
 *	vec4 /= float
 *	float = vec4 * vec4
 *
 *	we also allow
 *	vec4  = vec4 + float
 *	vec4  = vec4 - float
 *	vec4 += float
 *	vec4 -= float
 *	vec4  = float
 *	where every component of vec4 is in-/decremented by float
 */
#ifndef __XP_VEC4_
#define __XP_VEC4_

#include "XPmath.h"
#include "XPvec.h"

typedef struct vec4
{
	union
	{
		double	X,R,H;
	};
	union
	{
		double	Y,G,S;
	};
	union
	{
		double	Z,B,V;
	};
	double	A;

inline			vec4()
				{
					X=0;	Y=0;	Z=0;	A=0;
				}
				
inline			vec4(const double r, const double g, const double b, const double a)
				{
					R=r;	G=g;	B=b;	A=a;
				}
				
inline			vec4(const double arr[])
				{
					X=arr[0];	Y=arr[1];	Z=arr[2];	A=arr[3];
				}
				
				~vec4() {};

inline	vec4&	operator = (const vec4 a)
				{
					X=a.X;	Y=a.Y;	Z=a.Z;	A=a.A;
					return *this;
				}
				
inline	vec4&	operator = (const double a)
				{
					X=a;	Y=a;	Z=a;	A=a;
					return *this;
				}

inline	vec4&	operator += (const double a)
				{
					X+=a;	Y+=a;	Z+=a;	A+=a;
					return *this;
				}

inline	vec4&	operator += (const vec4 a)
				{
					X+=a.X;	Y+=a.Y;	Z+=a.Z;	A+=a.A;
					return *this;
				}

inline	vec4&	operator -= (const double a)
				{
					X-=a;	Y-=a;	Z-=a;	A-=a;
					return *this;
				}

inline	vec4&	operator -= (const vec4 a)
				{
					X-=a.X;	Y-=a.Y;	Z-=a.Z;	A-=a.A;
					return *this;
				}

inline	vec4&	operator /= (const double a)
				{
					X/=a;	Y/=a;	Z/=a;	A/=a;
					return *this;
				}
	
inline	vec4&	operator *= (const double a)
				{
					X*=a;	Y*=a;	Z*=a;	A*=a;
					return *this;
				}

inline	double	length()
				{
					return sqrt( X*X + Y*Y + Z*Z + A*A);
				}
};

inline vec4 operator + (const vec4 a, const vec4 b)
{
	return vec4(a.X+b.X , a.Y+b.Y , a.Z+b.Z , a.A+b.A);
}

inline vec4 operator - (const vec4 a, const vec4 b)
{
	return vec4(a.X-b.X , a.Y-b.Y , a.Z-b.Z , a.A-b.A);
}

inline vec4 operator + (const vec4 b, const double a)
{
	return vec4(b.X+a , b.Y+a , b.Z+a , b.A+a);
}

inline vec4 operator - (const vec4 b, const double a)
{
	return vec4(b.X-a , b.Y-a , b.Z-a , b.A-a);
}

inline vec4 operator * (const double a, const vec4 b)
{
	return vec4(b.X*a , b.Y*a , b.Z*a , b.A*a);
}

inline vec4 operator * (const vec4 b, const double a)
{
	return vec4(b.X*a , b.Y*a , b.Z*a , b.A*a);
}

inline vec4 operator / (const vec4 b, const double a)
{
	return vec4(b.X/a , b.Y/a , b.Z/a , b.A/a);
}

inline double operator * (const vec4 a, const vec4 b)
{
	return a.X*b.X + a.Y*b.Y + a.Z*b.Z + a.A*b.A;
}

inline vec4 operator -(const vec4 a)
{
	return vec4(-a.X , -a.Y , -a.Z , -a.A);
}

inline bool operator ==(const vec4 a, const vec4 b)
{
	return (a.X==b.X && a.Y==b.Y && a.Z==b.Z && a.A==b.A);
}

inline vec4 makevec4(const vec v3, const double a)
{
	return vec4(v3.R, v3.G, v3.B, a);
}

inline vec4 makevec4( const double r, const double g, const double b, const double a)
{
	return vec4(r,g,b,a);
}

inline vec makevec( const vec4 v4 )
{
	return vec( v4.R, v4.G, v4.B );
}

#endif
